import pyfie
import matplotlib.pyplot as plt

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 画像読み込み
pattern = pyfie.imread("fpm_pattern.png")

# 画像読み込み
target = pyfie.imread("fpm_target.png")

# マッチングパラメータ設定
max_results = 5
fpm_param = pyfie.F_FPM_MATCH(
    require_result_num=max_results,  # マッチング最大検出個数
    min_angle=-30,  # マッチング傾き範囲の最小値
    max_angle=30,  # マッチング傾き範囲の最大値
    min_scale=80,  # マッチングスケール範囲の最小値
    max_scale=120,  # マッチングスケール範囲の最大値
    coarse_comp_rate=3,  # 粗い精度でのマッチング時における特徴量の圧縮率
    coarse_err_wide=1,  # 粗い精度でのマッチング時におけるマッチングエッジ点の誤差範囲
    coarse_highcomp_threshold=60,  # 粗い精度でのマッチング時における高圧縮スコアしきい値
    coarse_lowcomp_threshold=60,  # 粗い精度でのマッチング時における低圧縮スコアしきい値
    refine_execute=True,  # 高精度ポーズ推定実行
    refine_threshold=70,  # 高精度ポーズ推定スコアしきい値
    refine_err_wide=1,  # 高精度ポーズ推定におけるエッジ点の誤差範囲
    polarity=pyfie.F_FPM_SAME_POLARITY  # 極性指定
)
matching_mode = pyfie.F_FPM_NORMAL_MODE  # マッチングモード

# 相関エッジパラメータ設定
edge_method = pyfie.F_FPM_CORR_EDGE
edge_param = pyfie.F_FPM_FEATURE(
    corr_edge=pyfie.F_EDGE_CORR_PARAMS(
        width=13,  # 相関エッジフィルタの長さ
        height=5,  # 相関エッジフィルタの高さ
        var_threshold=9.0,  # 分散しきい値
        sigmoid_k=1.0,  # シグモイド係数
        mag_threshold=160,  # 強度しきい値
        nms_length=6,  # 非極大抑制のフィルタ片幅
    )
)

# 回答オフセットとサーチウインドウを設定
offset = pyfie.DPNT_T(pattern.width / 2, pattern.height / 2)
win = pyfie.BOX_T((0, 0), (target.width - 1, target.height - 1))

# データ受け取りのための PyFIE データ型インスタンスを作成
results = pyfie.F_SEARCH_RESULT.ARRAY(max_results)
nresults = pyfie.INT()

# FPMオブジェクトを確保
hfpm = pyfie.fnFIE_fpm_alloc(
    pattern, None, offset, matching_mode, edge_method, edge_param, None
)
assert hfpm is not None

# サーチ実行
pyfie.fnFIE_fpm_matching(
    hfpm, target, None, win, edge_method, edge_param, fpm_param, None, results,
    nresults
)

# サーチ結果を表示
print("num results:", nresults)
for i in range(nresults):
    result = results[i]
    print("#{}: x = {:.2f}, y = {:.2f}, angle = {:.2f}, scale = {:.2f},"
          .format(i, result.x, result.y, result.q, result.s),
          "score ={}".format(result.score)
          )

# 画像オブジェクトのプロット
target.imshow()
# 結果をプロット
pyfie.mpltools.plot_fpm_results(hfpm, results)
# 結果をプロットした画像の保存
plt.savefig("fpm_result.png")
# 結果をプロットした画像の表示
pyfie.plot_show()
